@@ -1,4 +1,4 @@
-# Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -32,6 +32,8 @@ from abc import ABCMeta, abstractmethod, abstractproperty
 from decimal import Decimal
 from time import sleep
 from datetime import date, datetime, time, timedelta
+from inspect import signature
+import importlib
 import os
 import re
 import weakref
@@ -46,8 +48,8 @@ except:
 
 from .conversion import MySQLConverterBase
 from .constants import (ClientFlag, CharacterSet, CONN_ATTRS_DN,
-                        DEFAULT_CONFIGURATION, OPENSSL_CS_NAMES,
-                        TLS_CIPHER_SUITES, TLS_VERSIONS)
+                        DEFAULT_CONFIGURATION, DEPRECATED_TLS_VERSIONS,
+                        OPENSSL_CS_NAMES, TLS_CIPHER_SUITES, TLS_VERSIONS)
 from .optionfiles import MySQLOptionsParser
 from .utils import make_abc
 from . import errors
@@ -61,6 +63,9 @@ DUPLICATED_IN_LIST_ERROR = (
 TLS_VERSION_ERROR = ("The given tls_version: '{}' is not recognized as a valid "
                      "TLS protocol version (should be one of {}).")
 
+TLS_VERSION_DEPRECATED_ERROR = ("The given tls_version: '{}' are no longer "
+                                "allowed (should be one of {}).")
+
 TLS_VER_NO_SUPPORTED = ("No supported TLS protocol version found in the "
                         "'tls-versions' list '{}'. ")
 
@@ -105,6 +110,7 @@ class MySQLConnectionAbstract(object):
         self._ssl_disabled = DEFAULT_CONFIGURATION["ssl_disabled"]
         self._force_ipv6 = False
         self._oci_config_file = None
+        self._fido_callback = None
 
         self._use_unicode = True
         self._get_warnings = False
@@ -297,9 +303,6 @@ class MySQLConnectionAbstract(object):
                     tls_version = tls_ver.strip()
                     if tls_version == "":
                         continue
-                    elif tls_version not in TLS_VERSIONS:
-                        raise AttributeError(
-                            TLS_VERSION_ERROR.format(tls_version, TLS_VERSIONS))
                     elif tls_version in tls_versions:
                         raise AttributeError(
                             DUPLICATED_IN_LIST_ERROR.format(
@@ -314,10 +317,7 @@ class MySQLConnectionAbstract(object):
                     "At least one TLS protocol version must be specified in "
                     "'tls_versions' list.")
             for tls_ver in tls_version:
-                if tls_ver not in TLS_VERSIONS:
-                    raise AttributeError(
-                        TLS_VERSION_ERROR.format(tls_ver, TLS_VERSIONS))
-                elif tls_ver in tls_versions:
+                if tls_ver in tls_versions:
                     raise AttributeError(
                         DUPLICATED_IN_LIST_ERROR.format(
                             list="tls_versions", value=tls_ver))
@@ -325,9 +325,6 @@ class MySQLConnectionAbstract(object):
                     tls_versions.append(tls_ver)
         elif isinstance(tls_version, set):
             for tls_ver in tls_version:
-                if tls_ver not in TLS_VERSIONS:
-                    raise AttributeError(
-                        TLS_VERSION_ERROR.format(tls_ver, TLS_VERSIONS))
                 tls_versions.append(tls_ver)
         else:
             raise AttributeError(
@@ -338,11 +335,31 @@ class MySQLConnectionAbstract(object):
             raise AttributeError(
                 "At least one TLS protocol version must be specified "
                 "in 'tls_versions' list when this option is given.")
-        if tls_versions == ["TLSv1.3"] and not TLS_V1_3_SUPPORTED:
+
+        use_tls_versions = []
+        deprecated_tls_versions = []
+        invalid_tls_versions = []
+        for tls_ver in tls_versions:
+            if tls_ver in TLS_VERSIONS:
+                use_tls_versions.append(tls_ver)
+            if tls_ver in DEPRECATED_TLS_VERSIONS:
+                deprecated_tls_versions.append(tls_ver)
+            else:
+                invalid_tls_versions.append(tls_ver)
+
+        if use_tls_versions:
+            if use_tls_versions == ["TLSv1.3"] and not TLS_V1_3_SUPPORTED:
+                raise errors.NotSupportedError(
+                    TLS_VER_NO_SUPPORTED.format(tls_version, TLS_VERSIONS))
+            use_tls_versions.sort()
+            self._ssl["tls_versions"] = use_tls_versions
+        elif deprecated_tls_versions:
+            raise errors.NotSupportedError(
+                TLS_VERSION_DEPRECATED_ERROR.format(deprecated_tls_versions,
+                                                    TLS_VERSIONS))
+        elif invalid_tls_versions:
             raise AttributeError(
-                TLS_VER_NO_SUPPORTED.format(tls_version, TLS_VERSIONS))
-        tls_versions.sort()
-        self._ssl["tls_versions"] = tls_versions
+                TLS_VERSION_ERROR.format(tls_ver, TLS_VERSIONS))
 
     @property
     def user(self):
@@ -576,15 +593,9 @@ class MySQLConnectionAbstract(object):
                 )
             if "tls_versions" in self._ssl and \
                self._ssl["tls_versions"] is not None:
-                if self._ssl_disabled:
-                    raise AttributeError("The tls_versions option can not be "
-                                         "used along with ssl_disabled.")
                 self._validate_tls_versions()
 
             if "tls_ciphersuites" in self._ssl and self._ssl["tls_ciphersuites"] is not None:
-                if self._ssl_disabled:
-                    raise AttributeError("The tls_ciphersuites option can not "
-                                         "be used along with ssl_disabled.")
                 self._validate_tls_ciphersuites()
 
         if self._conn_attrs is None:
@@ -637,6 +648,33 @@ class MySQLConnectionAbstract(object):
                 raise errors.InterfaceError(KRB_SERVICE_PINCIPAL_ERROR.format(
                     error="is incorrectly formatted"))
 
+        if self._fido_callback:
+            # Import the callable if it's a str
+            if isinstance(self._fido_callback, str):
+                try:
+                    module, callback = self._fido_callback.rsplit(".", 1)
+                except ValueError:
+                    raise errors.ProgrammingError(
+                        f"No callable named '{self._fido_callback}'"
+                    )
+                try:
+                    module = importlib.import_module(module)
+                    self._fido_callback = getattr(module, callback)
+                except (AttributeError, ModuleNotFoundError) as err:
+                    raise errors.ProgrammingError(f"{err}")
+            # Check if it's a callable
+            if not callable(self._fido_callback):
+                raise errors.ProgrammingError(
+                    "Expected a callable for 'fido_callback'"
+                )
+            # Check the callable signature if has only 1 positional argument
+            params = len(signature(self._fido_callback).parameters)
+            if params != 1:
+                raise errors.ProgrammingError(
+                    "'fido_callback' requires 1 positional argument, but the "
+                    f"callback provided has {params}"
+                )
+
     def _add_default_conn_attrs(self):
         """Add the default connection attributes."""
         pass
