@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -29,16 +29,20 @@
 """Implementing communication with MySQL servers.
 """
 
+from decimal import Decimal
 from io import IOBase
+import datetime
+import getpass
+import logging
 import os
-import platform
 import socket
+import struct
 import time
+import warnings
 
 from .authentication import get_auth_plugin
-from .catch23 import PY2, isstr, UNICODE_TYPES
 from .constants import (
-    ClientFlag, ServerCmd, ServerFlag,
+    ClientFlag, ServerCmd, ServerFlag, FieldType,
     flag_is_set, ShutdownType, NET_BUFFER_LENGTH
 )
 
@@ -51,10 +55,15 @@ from .cursor import (
     MySQLCursorBufferedNamedTuple)
 from .network import MySQLUnixSocket, MySQLTCPSocket
 from .protocol import MySQLProtocol
-from .utils import int4store, linux_distribution
+from .utils import int1store, int4store, lc_int, get_platform
 from .abstracts import MySQLConnectionAbstract
 
 
+logging.getLogger(__name__).addHandler(logging.NullHandler())
+
+_LOGGER = logging.getLogger(__name__)
+
+
 class MySQLConnection(MySQLConnectionAbstract):
     """Connection to a MySQL Server"""
     def __init__(self, *args, **kwargs):
@@ -95,9 +104,12 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         self._ssl_active = False
         self._auth_plugin = None
+        self._krb_service_principal = None
         self._pool_config_version = None
+        self._query_attrs_supported = False
 
         self._columns_desc = []
+        self._mfa_nfactor = 1
 
         if kwargs:
             try:
@@ -110,21 +122,7 @@ class MySQLConnection(MySQLConnectionAbstract):
 
     def _add_default_conn_attrs(self):
         """Add the default connection attributes."""
-        if os.name == "nt":
-            if "64" in platform.architecture()[0]:
-                platform_arch = "x86_64"
-            elif "32" in platform.architecture()[0]:
-                platform_arch = "i386"
-            else:
-                platform_arch = platform.architecture()
-            os_ver = "Windows-{}".format(platform.win32_ver()[1])
-        else:
-            platform_arch = platform.machine()
-            if platform.system() == "Darwin":
-                os_ver = "{}-{}".format("macOS", platform.mac_ver()[0])
-            else:
-                os_ver = "-".join(linux_distribution()[0:2])
-
+        platform = get_platform()
         license_chunks = version.LICENSE.split(" ")
         if license_chunks[0] == "GPLv2":
             client_license = "GPL-2.0"
@@ -132,12 +130,15 @@ class MySQLConnection(MySQLConnectionAbstract):
             client_license = "Commercial"
         default_conn_attrs = {
             "_pid": str(os.getpid()),
-            "_platform": platform_arch,
+            "_platform": platform["arch"],
             "_source_host": socket.gethostname(),
             "_client_name": "mysql-connector-python",
             "_client_license": client_license,
-            "_client_version": ".".join([str(x) for x in version.VERSION[0:3]]),
-            "_os": os_ver}
+            "_client_version": ".".join(
+                [str(x) for x in version.VERSION[0:3]]
+            ),
+            "_os": platform["version"],
+        }
 
         self._conn_attrs.update((default_conn_attrs))
 
@@ -159,16 +160,27 @@ class MySQLConnection(MySQLConnectionAbstract):
             handshake['server_version_original'])
 
         if not handshake['capabilities'] & ClientFlag.SSL:
-            self._client_flags &= ~ClientFlag.SSL
+            if self._auth_plugin == "mysql_clear_password":
+                err_msg = ("Clear password authentication is not supported "
+                           "over insecure channels")
+                raise errors.InterfaceError(err_msg)
             if self._ssl.get('verify_cert'):
                 raise errors.InterfaceError("SSL is required but the server "
                                             "doesn't support it", errno=2026)
+            self._client_flags &= ~ClientFlag.SSL
         elif not self._ssl_disabled:
             self._client_flags |= ClientFlag.SSL
 
         if handshake['capabilities'] & ClientFlag.PLUGIN_AUTH:
             self.set_client_flags([ClientFlag.PLUGIN_AUTH])
 
+        if handshake['capabilities'] & ClientFlag.CLIENT_QUERY_ATTRIBUTES:
+            self._query_attrs_supported = True
+            self.set_client_flags([ClientFlag.CLIENT_QUERY_ATTRIBUTES])
+
+        if handshake['capabilities'] & ClientFlag.MULTI_FACTOR_AUTHENTICATION:
+            self.set_client_flags([ClientFlag.MULTI_FACTOR_AUTHENTICATION])
+
         self._handshake = handshake
 
     def _do_auth(self, username=None, password=None, database=None,
@@ -201,6 +213,24 @@ class MySQLConnection(MySQLConnectionAbstract):
                                        ssl_options.get('tls_versions'))
             self._ssl_active = True
 
+        if self._password1 and password != self._password1:
+            password = self._password1
+
+        _LOGGER.debug(
+                "# _do_auth(): user: %s", username)
+        _LOGGER.debug(
+                "# _do_auth(): self._auth_plugin: %s", self._auth_plugin)
+        if (self._auth_plugin.startswith("authentication_oci") or
+            (self._auth_plugin.startswith("authentication_kerberos") and
+             os.name == 'nt')) and not username:
+            username = getpass.getuser()
+            _LOGGER.debug(
+                "MySQL user is empty, OS user: %s will be used for %s",
+                username, self._auth_plugin)
+
+        _LOGGER.debug("# _do_auth(): user: %s", username)
+        _LOGGER.debug("# _do_auth(): password: %s", password)
+
         packet = self._protocol.make_auth(
             handshake=self._handshake,
             username=username, password=password, database=database,
@@ -224,6 +254,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         """
         auth = None
         new_auth_plugin = self._auth_plugin or self._handshake["auth_plugin"]
+        _LOGGER.debug("new_auth_plugin: %s", new_auth_plugin)
         packet = self._socket.recv()
         if packet[4] == 254 and len(packet) == 5:
             raise errors.NotSupportedError(
@@ -234,11 +265,10 @@ class MySQLConnection(MySQLConnectionAbstract):
             # AuthSwitchRequest
             (new_auth_plugin,
              auth_data) = self._protocol.parse_auth_switch_request(packet)
-            auth = get_auth_plugin(new_auth_plugin)(
-                auth_data, password=password, ssl_enabled=self._ssl_active)
-            response = auth.auth_response()
-            self._socket.send(response)
-            packet = self._socket.recv()
+            auth = get_auth_plugin(new_auth_plugin)(auth_data,
+                 username=self._user, password=password,
+                 ssl_enabled=self._ssl_active)
+            packet = self._auth_continue(auth, new_auth_plugin, auth_data)
 
         if packet[4] == 1:
             auth_data = self._protocol.parse_auth_more_data(packet)
@@ -252,10 +282,181 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         if packet[4] == 0:
             return self._handle_ok(packet)
+        elif packet[4] == 2:
+            return self._handle_mfa(packet)
         elif packet[4] == 255:
             raise errors.get_exception(packet)
         return None
 
+    def _handle_mfa(self, packet):
+        """Handle Multi Factor Authentication."""
+        self._mfa_nfactor += 1
+        if self._mfa_nfactor == 2:
+            password = self._password2
+        elif self._mfa_nfactor == 3:
+            password = self._password3
+        else:
+            raise errors.InterfaceError(
+                "Failed Multi Factor Authentication (invalid N factor)"
+            )
+
+        _LOGGER.debug("# MFA N Factor #%d", self._mfa_nfactor)
+
+        packet, auth_plugin = self._protocol.parse_auth_next_factor(
+            packet[4:]
+        )
+        auth = get_auth_plugin(auth_plugin)(
+            None,
+            username=self._user,
+            password=password,
+            ssl_enabled=self._ssl_active,
+        )
+        packet = self._auth_continue(auth, auth_plugin, packet)
+
+        if packet[4] == 1:
+            auth_data = self._protocol.parse_auth_more_data(packet)
+            auth = get_auth_plugin(auth_plugin)(
+                auth_data, password=password, ssl_enabled=self._ssl_active)
+            if auth_plugin == "caching_sha2_password":
+                response = auth.auth_response()
+                if response:
+                    self._socket.send(response)
+                packet = self._socket.recv()
+
+        if packet[4] == 0:
+            return self._handle_ok(packet)
+        elif packet[4] == 2:
+            return self._handle_mfa(packet)
+        elif packet[4] == 255:
+            raise errors.get_exception(packet)
+        return None
+
+    def _auth_continue(self, auth, auth_plugin, auth_data):
+        """Continue with the authentication."""
+        if auth_plugin == "authentication_ldap_sasl_client":
+            _LOGGER.debug("# auth_data: %s", auth_data)
+            response = auth.auth_response(self._krb_service_principal)
+        elif auth_plugin == "authentication_kerberos_client":
+            _LOGGER.debug("# auth_data: %s", auth_data)
+            response = auth.auth_response(auth_data)
+        elif auth_plugin == "authentication_oci_client":
+            _LOGGER.debug(
+                "# oci configuration file path: %s", self._oci_config_file
+            )
+            response = auth.auth_response(self._oci_config_file)
+        else:
+            response = auth.auth_response()
+
+        _LOGGER.debug("# request: %s size: %s", response, len(response))
+        self._socket.send(response)
+        packet = self._socket.recv()
+        _LOGGER.debug("# server response packet: %s", packet)
+        if (
+            auth_plugin == "authentication_ldap_sasl_client"
+            and len(packet) >= 6 and packet[5] == 114 and packet[6] == 61
+        ): # 'r' and '='
+            # Continue with sasl authentication
+            dec_response = packet[5:]
+            cresponse = auth.auth_continue(dec_response)
+            self._socket.send(cresponse)
+            packet = self._socket.recv()
+            if packet[5] == 118 and packet[6] == 61: # 'v' and '='
+                if auth.auth_finalize(packet[5:]):
+                    # receive packed OK
+                    packet = self._socket.recv()
+        elif (
+            auth_plugin == "authentication_ldap_sasl_client"
+            and auth_data == b'GSSAPI' and packet[4] != 255
+        ):
+            rcode_size = 5  # header size for the response status code.
+            _LOGGER.debug("# Continue with sasl GSSAPI authentication")
+            _LOGGER.debug("# response header: %s", packet[:rcode_size+1])
+            _LOGGER.debug("# response size: %s", len(packet))
+
+            _LOGGER.debug("# Negotiate a service request")
+            complete = False
+            tries = 0  # To avoid a infinite loop attempt no more than feedback messages
+            while not complete and tries < 5:
+                _LOGGER.debug("%s Attempt %s %s", "-" * 20, tries + 1, "-" * 20)
+                _LOGGER.debug("<< server response: %s", packet)
+                _LOGGER.debug("# response code: %s", packet[:rcode_size + 1])
+                step, complete = auth.auth_continue_krb(packet[rcode_size:])
+                _LOGGER.debug(" >> response to server: %s", step)
+                self._socket.send(step or b'')
+                packet = self._socket.recv()
+                tries += 1
+            if not complete:
+                raise errors.InterfaceError(
+                    "Unable to fulfill server request after %s attempts. "
+                    "Last server response: %s", tries, packet,
+                )
+            _LOGGER.debug(
+                " last GSSAPI response from server: %s length: %d",
+                packet,
+                len(packet),
+            )
+            last_step = auth.auth_accept_close_handshake(packet[rcode_size:])
+            _LOGGER.debug(
+                " >> last response to server: %s length: %d",
+                last_step,
+                len(last_step),
+            )
+            self._socket.send(last_step)
+            # Receive final handshake from server
+            packet = self._socket.recv()
+            _LOGGER.debug("<< final handshake from server: %s", packet)
+
+            # receive OK packet from server.
+            packet = self._socket.recv()
+            _LOGGER.debug("<< ok packet from server: %s", packet)
+        elif (
+            auth_plugin == "authentication_kerberos_client"
+            and packet[4] != 255
+        ):
+            rcode_size = 5  # Reader size for the response status code
+            _LOGGER.debug("# Continue with GSSAPI authentication")
+            _LOGGER.debug("# Response header: %s", packet[:rcode_size + 1])
+            _LOGGER.debug("# Response size: %s", len(packet))
+            _LOGGER.debug("# Negotiate a service request")
+            complete = False
+            tries = 0
+
+            while not complete and tries < 5:
+                _LOGGER.debug(
+                    "%s Attempt %s %s", "-" * 20, tries + 1, "-" * 20
+                )
+                _LOGGER.debug("<< Server response: %s", packet)
+                _LOGGER.debug(
+                    "# Response code: %s", packet[:rcode_size + 1]
+                )
+                token, complete = auth.auth_continue(packet[rcode_size:])
+                if token:
+                    self._socket.send(token)
+                if complete:
+                    break
+                packet = self._socket.recv()
+
+                _LOGGER.debug(">> Response to server: %s", token)
+                tries += 1
+
+            if not complete:
+                raise errors.InterfaceError(
+                    "Unable to fulfill server request after {} attempts. "
+                    "Last server response: {}".format(tries, packet)
+                )
+
+            _LOGGER.debug(
+                "Last response from server: %s length: %d",
+                packet,
+                len(packet),
+            )
+
+            # Receive OK packet from server.
+            packet = self._socket.recv()
+            _LOGGER.debug("<< Ok packet from server: %s", packet)
+
+        return packet
+
     def _get_connection(self, prtcls=None):
         """Get connection based on configuration
 
@@ -282,6 +483,11 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Raises on errors.
         """
+        if self._auth_plugin == "authentication_kerberos_client" and os.name != 'nt':
+            if not self._user:
+                cls = get_auth_plugin(self._auth_plugin)
+                self._user = cls.get_user_from_credentials()
+
         self._protocol = MySQLProtocol()
         self._socket = self._get_connection()
         try:
@@ -294,11 +500,27 @@ class MySQLConnection(MySQLConnectionAbstract):
             if self._client_flags & ClientFlag.COMPRESS:
                 self._socket.recv = self._socket.recv_compressed
                 self._socket.send = self._socket.send_compressed
+            self._socket.set_connection_timeout(None)
         except:
             # close socket
             self.close()
             raise
 
+        if (
+            not self._ssl_disabled
+            and hasattr(self._socket.sock, "version")
+            and callable(self._socket.sock.version)
+        ):
+            # Raise a deprecation warning if TLSv1 or TLSv1.1 is being used
+            tls_version = self._socket.sock.version()
+            if tls_version in ("TLSv1", "TLSv1.1"):
+                warn_msg = (
+                    f"This connection is using {tls_version} which is now "
+                    "deprecated and will be removed in a future release of "
+                    "MySQL Connector/Python"
+                )
+                warnings.warn(warn_msg, DeprecationWarning)
+
     def shutdown(self):
         """Shut down connection to MySQL Server.
         """
@@ -320,6 +542,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         except (AttributeError, errors.Error):
             pass  # Getting an exception would mean we are disconnected.
         self._socket.close_connection()
+        self._handshake = None
 
     disconnect = close
 
@@ -437,8 +660,34 @@ class MySQLConnection(MySQLConnectionAbstract):
 
     def _handle_load_data_infile(self, filename):
         """Handle a LOAD DATA INFILE LOCAL request"""
+        file_name = os.path.abspath(filename)
+        if os.path.islink(file_name):
+            raise errors.OperationalError("Use of symbolic link is not allowed")
+        if not self._allow_local_infile and \
+           not self._allow_local_infile_in_path:
+            raise errors.DatabaseError(
+                "LOAD DATA LOCAL INFILE file request rejected due to "
+                "restrictions on access.")
+        if not self._allow_local_infile and self._allow_local_infile_in_path:
+            # validate filename is inside of allow_local_infile_in_path path.
+            infile_path = os.path.abspath(self._allow_local_infile_in_path)
+            c_path = None
+            try:
+                c_path = os.path.commonpath([infile_path, file_name])
+            except ValueError as err:
+                err_msg = ("{} while loading file `{}` and path `{}` given"
+                           " in allow_local_infile_in_path")
+                raise errors.InterfaceError(
+                    err_msg.format(str(err), file_name, infile_path))
+
+            if c_path != infile_path:
+                err_msg = ("The file `{}` is not found in the given "
+                           "allow_local_infile_in_path {}")
+                raise errors.DatabaseError(
+                    err_msg.format(file_name,infile_path))
+
         try:
-            data_file = open(filename, 'rb')
+            data_file = open(file_name, 'rb')
             return self._handle_ok(self._send_data(data_file,
                                                    send_empty_packet=True))
         except IOError:
@@ -449,7 +698,7 @@ class MySQLConnection(MySQLConnectionAbstract):
                 raise errors.OperationalError(
                     "MySQL Connection not available.")
             raise errors.InterfaceError(
-                "File '{0}' could not be read".format(filename))
+                "File '{0}' could not be read".format(file_name))
         finally:
             try:
                 data_file.close()
@@ -475,10 +724,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         elif packet[4] == 0:
             return self._handle_ok(packet)
         elif packet[4] == 251:
-            if PY2:
-                filename = str(packet[5:])
-            else:
-                filename = packet[5:].decode()
+            filename = packet[5:].decode()
             return self._handle_load_data_infile(filename)
         elif packet[4] == 254:
             return self._handle_eof(packet)
@@ -593,10 +839,94 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a tuple()
         """
-        if not isinstance(query, bytes):
-            query = query.encode('utf-8')
-        result = self._handle_result(self._send_cmd(ServerCmd.QUERY, query))
+        if not isinstance(query, bytearray):
+            if isinstance(query, str):
+                query = query.encode('utf-8')
+            query = bytearray(query)
+        # Prepare query attrs
+        charset = self.charset if self.charset != "utf8mb4" else "utf8"
+        packet = bytearray()
+        if not self._query_attrs_supported and self._query_attrs:
+            warnings.warn(
+                "This version of the server does not support Query Attributes",
+                category=Warning)
+        if self._client_flags & ClientFlag.CLIENT_QUERY_ATTRIBUTES:
+            names = []
+            types = []
+            values = []
+            null_bitmap = [0] * ((len(self._query_attrs) + 7) // 8)
+            for pos, attr_tuple in enumerate(self._query_attrs):
+                value = attr_tuple[1]
+                flags = 0
+                if value is None:
+                    null_bitmap[(pos // 8)] |= 1 << (pos % 8)
+                    types.append(int1store(FieldType.NULL) +
+                                 int1store(flags))
+                    continue
+                elif isinstance(value, int):
+                    (packed, field_type,
+                     flags) = self._protocol._prepare_binary_integer(value)
+                    values.append(packed)
+                elif isinstance(value, str):
+                    value = value.encode(charset)
+                    values.append(lc_int(len(value)) + value)
+                    field_type = FieldType.VARCHAR
+                elif isinstance(value, bytes):
+                    values.append(lc_int(len(value)) + value)
+                    field_type = FieldType.BLOB
+                elif isinstance(value, Decimal):
+                    values.append(
+                        lc_int(len(str(value).encode(
+                            charset))) + str(value).encode(charset))
+                    field_type = FieldType.DECIMAL
+                elif isinstance(value, float):
+                    values.append(struct.pack('<d', value))
+                    field_type = FieldType.DOUBLE
+                elif isinstance(value, (datetime.datetime, datetime.date)):
+                    (packed, field_type) = \
+                       self._protocol._prepare_binary_timestamp(value)
+                    values.append(packed)
+                elif isinstance(value, (datetime.timedelta, datetime.time)):
+                    (packed, field_type) = \
+                       self._protocol._prepare_binary_time(value)
+                    values.append(packed)
+                else:
+                    raise errors.ProgrammingError(
+                        "MySQL binary protocol can not handle "
+                        "'{classname}' objects".format(
+                            classname=value.__class__.__name__))
+                types.append(int1store(field_type) +
+                             int1store(flags))
+                name = attr_tuple[0].encode(charset)
+                names.append(lc_int(len(name)) + name)
 
+            # int<lenenc>    parameter_count    Number of parameters
+            packet.extend(lc_int(len(self._query_attrs)))
+            # int<lenenc>    parameter_set_count    Number of parameter sets.
+            # Currently always 1
+            packet.extend(lc_int(1))
+            if values:
+                packet.extend(
+                    b''.join([struct.pack('B', bit) for bit in null_bitmap]) +
+                    int1store(1))
+                for _type, name in zip(types, names):
+                    packet.extend(_type)
+                    packet.extend(name)
+
+                for value in values:
+                    packet.extend(value)
+
+        packet.extend(query)
+        query = bytes(packet)
+        try:
+            result = self._handle_result(self._send_cmd(ServerCmd.QUERY, query))
+        except errors.ProgrammingError as err:
+            if err.errno == 3948 and \
+               "Loading local data is disabled" in err.msg:
+                err_msg = ("LOAD DATA LOCAL INFILE file request rejected due "
+                           "to restrictions on access.")
+                raise errors.DatabaseError(err_msg)
+            raise
         if self._have_next_result:
             raise errors.InterfaceError(
                 'Use cmd_query_iter for statements with multiple queries.')
@@ -620,13 +950,23 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a generator.
         """
+        packet = bytearray()
         if not isinstance(statements, bytearray):
-            if isstr(statements) and isinstance(statements, UNICODE_TYPES):
+            if isinstance(statements, str):
                 statements = statements.encode('utf8')
             statements = bytearray(statements)
 
+        if self._client_flags & ClientFlag.CLIENT_QUERY_ATTRIBUTES:
+            # int<lenenc>    parameter_count    Number of parameters
+            packet.extend(lc_int(0))
+            # int<lenenc>    parameter_set_count    Number of parameter sets.
+            # Currently always 1
+            packet.extend(lc_int(1))
+
+        packet.extend(statements)
+        query = bytes(packet)
         # Handle the first query result
-        yield self._handle_result(self._send_cmd(ServerCmd.QUERY, statements))
+        yield self._handle_result(self._send_cmd(ServerCmd.QUERY, query))
 
         # Handle next results, if any
         while self._have_next_result:
@@ -735,7 +1075,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         return self._handle_ok(self._send_cmd(ServerCmd.PING))
 
     def cmd_change_user(self, username='', password='', database='',
-                        charset=45):
+                        charset=45, password1='', password2='', password3='',
+                        oci_config_file=''):
         """Change the current logged in user
 
         This method allows to change the current logged in user information.
@@ -743,20 +1084,33 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a dict()
         """
+        self._mfa_nfactor = 1
+        self._user = username
+        self._password = password
+        self._password1 = password1
+        self._password2 = password2
+        self._password3 = password3
+
+        if self._password1 and password != self._password1:
+            password = self._password1
+
         self.handle_unread_result()
 
         if self._compress:
             raise errors.NotSupportedError("Change user is not supported with "
                                            "compression.")
-
         packet = self._protocol.make_change_user(
             handshake=self._handshake,
             username=username, password=password, database=database,
             charset=charset, client_flags=self._client_flags,
             ssl_enabled=self._ssl_active,
-            auth_plugin=self._auth_plugin)
+            auth_plugin=self._auth_plugin,
+            conn_attrs=self._conn_attrs)
         self._socket.send(packet, 0, 0)
 
+        if oci_config_file:
+            self._oci_config_file = oci_config_file
+
         ok_packet = self._auth_switch_request(username, password)
 
         try:
@@ -796,6 +1150,13 @@ class MySQLConnection(MySQLConnectionAbstract):
             return False  # This method does not raise
         return True
 
+    def set_allow_local_infile_in_path(self, path):
+        """set local_infile_in_path
+
+        Set allow_local_infile_in_path.
+        """
+        self._allow_local_infile_in_path = path
+
     def reset_session(self, user_variables=None, session_variables=None):
         """Clears the current active session
 
@@ -815,11 +1176,15 @@ class MySQLConnection(MySQLConnectionAbstract):
         if not self.is_connected():
             raise errors.OperationalError("MySQL Connection not available.")
 
-        try:
-            self.cmd_reset_connection()
-        except errors.NotSupportedError:
-            self.cmd_change_user(self._user, self._password,
-                                 self._database, self._charset_id)
+        if not self.cmd_reset_connection():
+            try:
+                self.cmd_change_user(self._user, self._password,
+                                     self._database, self._charset_id,
+                                     self._password1, self._password2,
+                                     self._password3,
+                                     self._oci_config_file)
+            except errors.ProgrammingError:
+                self.reconnect()
 
         cur = self.cursor()
         if user_variables:
@@ -884,10 +1249,9 @@ class MySQLConnection(MySQLConnectionAbstract):
     @property
     def connection_id(self):
         """MySQL connection ID"""
-        try:
-            return self._handshake['server_threadid']
-        except KeyError:
-            return None
+        if self._handshake:
+            return self._handshake.get("server_threadid")
+        return None
 
     def cursor(self, buffered=None, raw=None, prepared=None, cursor_class=None,
                dictionary=None, named_tuple=None):
@@ -1091,10 +1455,20 @@ class MySQLConnection(MySQLConnectionAbstract):
                     self.cmd_stmt_send_long_data(statement_id, param_id,
                                                  data[param_id])
                     long_data_used[param_id] = (binary,)
-
-        execute_packet = self._protocol.make_stmt_execute(
-            statement_id, data, tuple(parameters), flags,
-            long_data_used, self.charset)
+        if not self._query_attrs_supported and self._query_attrs:
+            warnings.warn(
+                "This version of the server does not support Query Attributes",
+                category=Warning)
+        if self._client_flags & ClientFlag.CLIENT_QUERY_ATTRIBUTES:
+            execute_packet = self._protocol.make_stmt_execute(
+                statement_id, data, tuple(parameters), flags,
+                long_data_used, self.charset, self._query_attrs,
+                self._converter_str_fallback)
+        else:
+            execute_packet = self._protocol.make_stmt_execute(
+                statement_id, data, tuple(parameters), flags,
+                long_data_used, self.charset,
+                converter_str_fallback=self._converter_str_fallback)
         packet = self._send_cmd(ServerCmd.STMT_EXECUTE, packet=execute_packet)
         result = self._handle_binary_result(packet)
         return result
@@ -1157,17 +1531,17 @@ class MySQLConnection(MySQLConnectionAbstract):
     def cmd_reset_connection(self):
         """Resets the session state without re-authenticating
 
-        Works only for MySQL server 5.7.3 or later.
-        The result is a dictionary with OK packet information.
+        Reset command only works on MySQL server 5.7.3 or later.
+        The result is True for a successful reset otherwise False.
 
-        Returns a dict()
+        Returns bool
         """
-        if self._server_version < (5, 7, 3):
-            raise errors.NotSupportedError("MySQL version 5.7.2 and "
-                                           "earlier does not support "
-                                           "COM_RESET_CONNECTION.")
-        self._handle_ok(self._send_cmd(ServerCmd.RESET_CONNECTION))
-        self._post_connection()
+        try:
+            self._handle_ok(self._send_cmd(ServerCmd.RESET_CONNECTION))
+            self._post_connection()
+            return True
+        except (errors.NotSupportedError, errors.OperationalError):
+            return False
 
     def handle_unread_result(self):
         """Check whether there is an unread result"""
