@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Oracle and/or its affiliates.
+# Copyright (c) 2020, 2021, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -54,7 +54,7 @@ from django.utils import dateparse, timezone
 
 try:
     import mysql.connector
-    from mysql.connector.conversion import MySQLConverter, MySQLConverterBase
+    from mysql.connector.conversion import MySQLConverter
 except ImportError as err:
     raise ImproperlyConfigured(
         "Error loading mysql.connector module: {0}".format(err))
@@ -276,13 +276,22 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     def __init__(self, *args, **kwargs):
         super(DatabaseWrapper, self).__init__(*args, **kwargs)
 
-        try:
-            self._use_pure = self.settings_dict['OPTIONS']['use_pure']
-        except KeyError:
-            self._use_pure = not HAVE_CEXT
-
-        self.converter = DjangoMySQLConverter() \
-            if self.use_pure else DjangoCMySQLConverter()
+        options = self.settings_dict.get('OPTIONS')
+        if options:
+            self._use_pure = options.get('use_pure', not HAVE_CEXT)
+            converter_class = options.get(
+                'converter_class',
+                DjangoMySQLConverter,
+            )
+            if not issubclass(converter_class, DjangoMySQLConverter):
+                raise ProgrammingError(
+                    'Converter class should be a subclass of '
+                    'mysql.connector.django.base.DjangoMySQLConverter'
+                )
+            self.converter = converter_class()
+        else:
+             self.converter = DjangoMySQLConverter()
+             self._use_pure = not HAVE_CEXT
 
     def __getattr__(self, attr):
         if attr.startswith("mysql_is"):
@@ -328,9 +337,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         return kwargs
 
     def get_new_connection(self, conn_params):
-        if not self.use_pure:
-            conn_params['converter_class'] = DjangoCMySQLConverter
-        else:
+        if not 'converter_class' in conn_params:
             conn_params['converter_class'] = DjangoMySQLConverter
         cnx = mysql.connector.connect(**conn_params)
 
@@ -455,6 +462,30 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             return check_constraints
         return {}
 
+    @cached_property
+    def mysql_server_data(self):
+        with self.temporary_connection() as cursor:
+            # Select some server variables and test if the time zone
+            # definitions are installed. CONVERT_TZ returns NULL if 'UTC'
+            # timezone isn't loaded into the mysql.time_zone table.
+            cursor.execute("""
+                SELECT VERSION(),
+                       @@sql_mode,
+                       @@default_storage_engine,
+                       @@sql_auto_is_null,
+                       @@lower_case_table_names,
+                       CONVERT_TZ('2001-01-01 01:00:00', 'UTC', 'UTC') IS NOT NULL
+            """)
+            row = cursor.fetchone()
+        return {
+            'version': row[0],
+            'sql_mode': row[1],
+            'default_storage_engine': row[2],
+            'sql_auto_is_null': bool(row[3]),
+            'lower_case_table_names': bool(row[4]),
+            'has_zoneinfo_database': bool(row[5]),
+        }
+
     @cached_property
     def mysql_server_info(self):
         with self.temporary_connection() as cursor:
@@ -478,11 +509,10 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     @property
     def use_pure(self):
         return self._use_pure
-        # return not HAVE_CEXT or self._use_pure
 
 
 class DjangoMySQLConverter(MySQLConverter):
-    """Custom converter for Django for MySQLConnection"""
+    """Custom converter for Django."""
     def _TIME_to_python(self, value, dsc=None):
         """Return MySQL TIME data type as datetime.time()
 
@@ -516,28 +546,3 @@ class DjangoMySQLConverter(MySQLConverter):
 
     def _safebytes_to_mysql(self, value):
         return self._bytes_to_mysql(value)
-
-
-class DjangoCMySQLConverter(MySQLConverterBase):
-    """Custom converter for Django for CMySQLConnection"""
-    def _TIME_to_python(self, value, dsc=None):
-        """Return MySQL TIME data type as datetime.time()
-
-        Returns datetime.time()
-        """
-        return dateparse.parse_time(str(value))
-
-    def __DATETIME_to_python(self, value, dsc=None):
-        """Connector/Python always returns naive datetime.datetime
-
-        Connector/Python always returns naive timestamps since MySQL has
-        no time zone support. Since Django needs non-naive, we need to add
-        the UTC time zone.
-
-        Returns datetime.datetime()
-        """
-        if not value:
-            return None
-        if settings.USE_TZ and timezone.is_naive(value):
-            value = value.replace(tzinfo=timezone.utc)
-        return value
