@@ -33,9 +33,13 @@ import json
 import logging
 import ssl
 
-from . import constants
-from .compat import (INT_TYPES, STRING_TYPES, JSONDecodeError, urlparse,
-                     unquote, parse_qsl)
+from urllib.parse import parse_qsl, unquote, urlparse
+
+try:
+    from json.decoder import JSONDecodeError
+except ImportError:
+    JSONDecodeError = ValueError
+
 from .connection import Client, Session
 from .constants import (Auth, LockContention, OPENSSL_CS_NAMES, SSLMode,
                         TLS_VERSIONS, TLS_CIPHER_SUITES)
@@ -53,9 +57,9 @@ from .statement import (Statement, FilterableStatement, SqlStatement,
                         DeleteStatement, UpdateStatement,
                         CreateCollectionIndexStatement, Expr, ReadStatement,
                         WriteStatement)
-
 from .expr import ExprParser as expr
 
+
 _SPLIT_RE = re.compile(r",(?![^\(\)]*\))")
 _PRIORITY_RE = re.compile(r"^\(address=(.+),priority=(\d+)\)$", re.VERBOSE)
 _ROUTER_RE = re.compile(r"^\(address=(.+)[,]*\)$", re.VERBOSE)
@@ -73,13 +77,18 @@ DUPLICATED_IN_LIST_ERROR = (
     "The '{list}' list must not contain repeated values, the value "
     "'{value}' is duplicated.")
 
-TLS_VERSION_ERROR = ("The given tls-version: '{}' is not recognized as a valid "
-                     "TLS protocol version (should be one of {}).")
+TLS_VERSION_ERROR = ("The given tls-version: '{}' is not recognized as a "
+                     "valid TLS protocol version (should be one of {}).")
+
+TLS_VERSION_DEPRECATED_ERROR = ("The given tls_version: '{}' are no longer "
+                                "allowed (should be one of {}).")
 
 TLS_VER_NO_SUPPORTED = ("No supported TLS protocol version found in the "
                         "'tls-versions' list '{}'. ")
 
-TLS_VERSIONS = ["TLSv1", "TLSv1.1", "TLSv1.2", "TLSv1.3"]
+TLS_VERSIONS = ["TLSv1.2", "TLSv1.3"]
+
+DEPRECATED_TLS_VERSIONS = ["TLSv1", "TLSv1.1"]
 
 TLS_V1_3_SUPPORTED = False
 if hasattr(ssl, "HAS_TLSv1_3") and ssl.HAS_TLSv1_3:
@@ -149,8 +158,7 @@ def _parse_connection_uri(uri):
         MySQL server/farm.
 
     Raises:
-        :class:`mysqlx.InterfaceError`: If contains a duplicate option or
-                                        URI scheme is not valid.
+        :class:`mysqlx.InterfaceError`: If contains a invalid option.
     """
     settings = {"schema": ""}
 
@@ -188,8 +196,6 @@ def _parse_connection_uri(uri):
         opt = key.replace("_", "-").lower()
         if opt in invalid_options:
             raise InterfaceError("Invalid option: '{0}'".format(key))
-        if opt in settings:
-            raise InterfaceError("Duplicate option: '{0}'".format(key))
         if opt in _SSL_OPTS:
             settings[opt] = unquote(val.strip("()"))
         else:
@@ -233,9 +239,6 @@ def _validate_settings(settings):
         except (AttributeError, ValueError):
             raise InterfaceError("Invalid SSL Mode '{0}'"
                                  "".format(settings["ssl-mode"]))
-        if settings["ssl-mode"] == SSLMode.DISABLED and \
-            any(key in settings for key in _SSL_OPTS):
-            raise InterfaceError("SSL options used with ssl-mode 'disabled'")
 
     if "ssl-crl" in settings and not "ssl-ca" in settings:
         raise InterfaceError("CA Certificate not provided")
@@ -246,7 +249,7 @@ def _validate_settings(settings):
         in [SSLMode.VERIFY_IDENTITY, SSLMode.VERIFY_CA]:
         raise InterfaceError("Cannot verify Server without CA")
     if "ssl-ca" in settings and settings.get("ssl-mode") \
-        not in [SSLMode.VERIFY_IDENTITY, SSLMode.VERIFY_CA]:
+        not in [SSLMode.VERIFY_IDENTITY, SSLMode.VERIFY_CA, SSLMode.DISABLED]:
         raise InterfaceError("Must verify Server if CA is provided")
 
     if "auth" in settings:
@@ -266,7 +269,7 @@ def _validate_settings(settings):
         settings["compression"] = compression
 
     if "compression-algorithms" in settings:
-        if isinstance(settings["compression-algorithms"], STRING_TYPES):
+        if isinstance(settings["compression-algorithms"], str):
             compression_algorithms = \
                 settings["compression-algorithms"].strip().strip("[]")
             if compression_algorithms:
@@ -285,9 +288,9 @@ def _validate_settings(settings):
 
     if "connect-timeout" in settings:
         try:
-            if isinstance(settings["connect-timeout"], STRING_TYPES):
+            if isinstance(settings["connect-timeout"], str):
                 settings["connect-timeout"] = int(settings["connect-timeout"])
-            if not isinstance(settings["connect-timeout"], INT_TYPES) \
+            if not isinstance(settings["connect-timeout"], int) \
                or settings["connect-timeout"] < 0:
                 raise ValueError
         except ValueError:
@@ -362,7 +365,7 @@ def _validate_connection_attributes(settings):
 
     conn_attrs = settings["connection-attributes"]
 
-    if isinstance(conn_attrs, STRING_TYPES):
+    if isinstance(conn_attrs, str):
         if conn_attrs == "":
             settings["connection-attributes"] = {}
             return
@@ -394,7 +397,7 @@ def _validate_connection_attributes(settings):
     elif isinstance(conn_attrs, dict):
         for attr_name in conn_attrs:
             attr_value = conn_attrs[attr_name]
-            if not isinstance(attr_value, STRING_TYPES):
+            if not isinstance(attr_value, str):
                 attr_value = repr(attr_value)
             attributes[attr_name] = attr_value
     elif isinstance(conn_attrs, bool) or conn_attrs in [0, 1]:
@@ -429,7 +432,7 @@ def _validate_connection_attributes(settings):
             attr_value = attributes[attr_name]
 
             # Validate name type
-            if not isinstance(attr_name, STRING_TYPES):
+            if not isinstance(attr_name, str):
                 raise InterfaceError("Attribute name '{}' must be a string"
                                      "type".format(attr_name))
             # Validate attribute name limit 32 characters
@@ -443,7 +446,7 @@ def _validate_connection_attributes(settings):
                                      "".format(attr_name))
 
             # Validate value type
-            if not isinstance(attr_value, STRING_TYPES):
+            if not isinstance(attr_value, str):
                 raise InterfaceError("Attribute '{}' value: '{}' must "
                                      "be a string type"
                                      "".format(attr_name, attr_value))
@@ -471,7 +474,7 @@ def _validate_tls_versions(settings):
 
     tls_versions_settings = settings["tls-versions"]
 
-    if isinstance(tls_versions_settings, STRING_TYPES):
+    if isinstance(tls_versions_settings, str):
         if not (tls_versions_settings.startswith("[") and
                 tls_versions_settings.endswith("]")):
             raise InterfaceError("tls-versions must be a list, found: '{}'"
@@ -482,9 +485,6 @@ def _validate_tls_versions(settings):
                 tls_version = tls_ver.strip()
                 if tls_version == "":
                     continue
-                if tls_version not in TLS_VERSIONS:
-                    raise InterfaceError(TLS_VERSION_ERROR.format(tls_version,
-                                                                  TLS_VERSIONS))
                 else:
                     if tls_version in tls_versions:
                         raise InterfaceError(
@@ -496,10 +496,7 @@ def _validate_tls_versions(settings):
             raise InterfaceError("At least one TLS protocol version must be "
                                  "specified in 'tls-versions' list.")
         for tls_ver in tls_versions_settings:
-            if tls_ver not in TLS_VERSIONS:
-                raise InterfaceError(TLS_VERSION_ERROR.format(tls_ver,
-                                                              TLS_VERSIONS))
-            elif tls_ver in tls_versions:
+            if tls_ver in tls_versions:
                 raise InterfaceError(
                     DUPLICATED_IN_LIST_ERROR.format(list="tls_versions",
                                                     value=tls_ver))
@@ -508,11 +505,7 @@ def _validate_tls_versions(settings):
 
     elif isinstance(tls_versions_settings, set):
         for tls_ver in tls_versions_settings:
-            if tls_ver not in TLS_VERSIONS:
-                raise InterfaceError(TLS_VERSION_ERROR.format(tls_ver,
-                                                              TLS_VERSIONS))
-            else:
-                tls_versions.append(tls_ver)
+            tls_versions.append(tls_ver)
     else:
         raise InterfaceError("tls-versions should be a list with one or more "
                              "of versions in {}. found: '{}'"
@@ -522,11 +515,30 @@ def _validate_tls_versions(settings):
         raise InterfaceError("At least one TLS protocol version must be "
                              "specified in 'tls-versions' list.")
 
-    if tls_versions == ["TLSv1.3"] and not TLS_V1_3_SUPPORTED:
-            raise InterfaceError(
-                TLS_VER_NO_SUPPORTED.format(tls_versions, TLS_VERSIONS))
+    use_tls_versions = []
+    deprecated_tls_versions = []
+    not_tls_versions = []
+    for tls_ver in tls_versions:
+        if tls_ver in TLS_VERSIONS:
+            use_tls_versions.append(tls_ver)
+        if tls_ver in DEPRECATED_TLS_VERSIONS:
+            deprecated_tls_versions.append(tls_ver)
+        else:
+            not_tls_versions.append(tls_ver)
 
-    settings["tls-versions"] = tls_versions
+    if use_tls_versions:
+        if use_tls_versions == ["TLSv1.3"] and not TLS_V1_3_SUPPORTED:
+            raise NotSupportedError(
+                TLS_VER_NO_SUPPORTED.format(tls_versions, TLS_VERSIONS))
+        use_tls_versions.sort()
+        settings["tls-versions"] = use_tls_versions
+    elif deprecated_tls_versions:
+        raise NotSupportedError(
+            TLS_VERSION_DEPRECATED_ERROR.format(deprecated_tls_versions,
+                                                TLS_VERSIONS))
+    elif not_tls_versions:
+        raise InterfaceError(
+            TLS_VERSION_ERROR.format(tls_ver, TLS_VERSIONS))
 
 
 def _validate_tls_ciphersuites(settings):
@@ -544,7 +556,7 @@ def _validate_tls_ciphersuites(settings):
 
     tls_ciphersuites_settings = settings["tls-ciphersuites"]
 
-    if isinstance(tls_ciphersuites_settings, STRING_TYPES):
+    if isinstance(tls_ciphersuites_settings, str):
         if not (tls_ciphersuites_settings.startswith("[") and
                 tls_ciphersuites_settings.endswith("]")):
             raise InterfaceError("tls-ciphersuites must be a list, found: '{}'"
@@ -632,7 +644,7 @@ def _get_connection_settings(*args, **kwargs):
     """
     settings = {}
     if args:
-        if isinstance(args[0], STRING_TYPES):
+        if isinstance(args[0], str):
             settings = _parse_connection_uri(args[0])
         elif isinstance(args[0], dict):
             for key, val in args[0].items():
@@ -715,15 +727,15 @@ def get_client(connection_string, options_string):
 
     .. versionadded:: 8.0.13
     """
-    if not isinstance(connection_string, (STRING_TYPES, dict)):
+    if not isinstance(connection_string, (str, dict)):
         raise InterfaceError("connection_data must be a string or dict")
 
     settings_dict = _get_connection_settings(connection_string)
 
-    if not isinstance(options_string, (STRING_TYPES, dict)):
+    if not isinstance(options_string, (str, dict)):
         raise InterfaceError("connection_options must be a string or dict")
 
-    if isinstance(options_string, STRING_TYPES):
+    if isinstance(options_string, str):
         try:
             options_dict = json.loads(options_string)
         except JSONDecodeError:
@@ -786,5 +798,6 @@ __all__ = [
     "DbDoc", "Statement", "FilterableStatement", "SqlStatement",
     "FindStatement", "AddStatement", "RemoveStatement", "ModifyStatement",
     "SelectStatement", "InsertStatement", "DeleteStatement", "UpdateStatement",
-    "CreateCollectionIndexStatement", "Expr",
+    "ReadStatement", "WriteStatement", "CreateCollectionIndexStatement",
+    "Expr",
 ]
